package SPVM::Net::SSLeay::EVP;



1;

=head1 Name

SPVM::Net::SSLeay::EVP - EVP Name Space in OpenSSL

=head1 Description

Net::SSLeay::EVP class in L<SPVM> represents L<EVP|https://docs.openssl.org/3.1/man3/EVP_DigestInit> Name Space in OpenSSL.

=head1 Usage

  use Net::SSLeay::EVP;

=head1 Class Methods


=head1 See Also

=over 2

=item * L<Net::SSLeay::EVP_MD|SPVM::Net::EVP_MD>

=item * L<Net::SSLeay|SPVM::Net::SSLeay>

=back

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License

