// Copyright (c) 2024 Yuki Kimoto
// MIT License

#include "spvm_native.h"

#include <openssl/ssl.h>
#include <openssl/err.h>

static const char* FILE_NAME = "Net/SSLeay/EVP_MD.c";

int32_t SPVM__Net__SSLeay__EVP_MD__foo(SPVM_ENV* env, SPVM_VALUE* stack) {
  
  return 0;
}


