package SPVM::Net::SSLeay::SSL_CIPHER;



1;

=head1 Name

SPVM::Net::SSLeay::SSL_CIPHER - SSL_CIPHER Data Strucutre in OpenSSL

=head1 Description

Net::SSLeay::SSL_CIPHER class in L<SPVM> represents L<SSL_CIPHER|https://docs.openssl.org/1.0.2/man3/SSL_get_current_cipher/> Data Strucutre in OpenSSL

=head1 Usage

  use Net::SSLeay::SSL_CIPHER;

=head1 Instance Methods



=head1 See Also

=over 2

=item * L<Net::SSLeay|SPVM::Net::SSLeay>

=back

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License

