package SPVM::Net::SSLeay::EVP_MD;



1;

=head1 Name

SPVM::Net::SSLeay::EVP_MD - EVP_MD Data Structure in OpenSSL

=head1 Description

Net::SSLeay::EVP_MD class in L<SPVM> represents L<EVP_MD|https://docs.openssl.org/3.0/man3/EVP_sha224/> Data Structure in OpenSSL

=head1 Usage

  use Net::SSLeay::EVP_MD;

=head1 See Also

=over 2

=item * L<Net::SSLeay|SPVM::Net::EVP>

=item * L<Net::SSLeay|SPVM::Net::SSLeay>

=back

=head1 Copyright & License

Copyright (c) 2024 Yuki Kimoto

MIT License

